@@ -12,11 +12,12 @@ from .formatter import *
 from .shell import *
 from .irc import *
 from .nuts import *
+from .tinymux import *
 
 import PIL
 from collections import Sequence
 
-VERSION = "1.4.1"
+VERSION = "1.5"
 
 def scale(image, width, height):
     """
