@@ -5,10 +5,14 @@
 # Lars Jørgen Solberg <supersolberg@gmail.com> 2014
 #
 
+from __future__ import print_function
+from __future__ import division
+from __future__ import unicode_literals
+
 import shellpic
 
 import os
-import StringIO
+import io
 import termios
 import sys
 import re
@@ -67,41 +71,50 @@ class Shell(shellpic.Formatter):
         # disable echo and make sure input is sent char-by-char
         new_attrs[3] &= ~termios.ECHO
         new_attrs[3] &= ~termios.ICANON
-        termios.tcsetattr(sys.stdin, termios.TCSANOW, new_attrs)
 
-        # ask for the cursor position
-        print chr(27) +'[6n',
-        # read the response from the terminal
-        response = ''
-        while True:
-            char = sys.stdin.read(1)
-            response += char
-            if char == 'R':
-                break
+        try:
+            termios.tcsetattr(sys.stdin, termios.TCSANOW, new_attrs)
+            termios.tcsetattr(sys.stdout, termios.TCSANOW, new_attrs)
 
-        # restore terminal attributes
-        termios.tcsetattr(sys.stdin, termios.TCSANOW, old_attrs)
+            # ask for the cursor position
+            sys.stdout.write(chr(27) + u'[6n')
+            if sys.version_info[0] == 3:
+                sys.stdout.flush()
+            # read the response from the terminal
+            response = u''
+
+            while True:
+                char = sys.stdin.read(1)
+                response += char
+                if char == u'R':
+                    break
+
+            # parse the response and return
+            m = re.match(r'\033\[(\d+);(\d+)R', response)
+            y, x = m.groups()
+
+        finally:
+            # restore terminal attributes
+            termios.tcsetattr(sys.stdin, termios.TCSANOW, old_attrs)
+            termios.tcsetattr(sys.stdout, termios.TCSANOW, old_attrs)
 
-        # parse the response and return
-        m = re.match(r'\033\[(\d+);(\d+)R', response)
-        y, x = m.groups()
         return [int(x) - 1, int(y) - 1]
 
     def move_cursor(self,  pos_x, pos_y):
-        return "{0}[{1};{2}f".format(chr(27), self._origin[1] + pos_y,
+        return u"{0}[{1};{2}f".format(chr(27), self._origin[1] + pos_y,
                                   self._origin[0] + pos_x)
 
     @staticmethod
     def save_cursor():
-        return "{0}[s".format(chr(27))
+        return u"{0}[s".format(chr(27))
 
     @staticmethod
     def restore_cursor():
-        return "{0}[r".format(chr(27))
+        return u"{0}[r".format(chr(27))
 
     @staticmethod
     def clear_screen():
-        return "[{0}[2J".format(chr(27))
+        return u"[{0}[2J".format(chr(27))
 
     @classmethod
     def colorcode(cls, bgcolor, fgcolor):
@@ -161,7 +174,7 @@ class Shell(shellpic.Formatter):
         if dispose:
             assert dispose.mode == 'RGBA'
 
-        file_str = StringIO.StringIO()
+        file_str = io.StringIO()
 
         width, height = image.size
         # since we put two pixels on top of each other in each character position
@@ -170,7 +183,7 @@ class Shell(shellpic.Formatter):
 
         if not self._prev_frame:
             # create some empty space to draw on
-            file_str.write('\n' * (padded_height / 2))
+            file_str.write(u'\n' * (padded_height // 2))
 
             # find out where we shold put the top left pixel if we are
             # in a terminal. Just use (0, 0) if we are piping to or
@@ -178,8 +191,8 @@ class Shell(shellpic.Formatter):
             if os.isatty(sys.stdin.fileno()) and os.isatty(sys.stdout.fileno()):
                 x, y = self.probe_cursor_pos()
                 term_width, term_height = self.dimentions()
-                if y + (padded_height / 2) > term_height:
-                    adjust = (y + (padded_height / 2)) - term_height
+                if y + (padded_height // 2) > term_height:
+                    adjust = (y + (padded_height // 2)) - term_height
                     self._origin = x, y - adjust
                 else:
                     self._origin = x, y
@@ -214,10 +227,10 @@ class Shell(shellpic.Formatter):
         for y in range(0, height - 1, 2):
             for x in range(0, width):
                 if self.need_repaint(pixels, x, y):
-                    file_str.write(self.move_cursor(x, y / 2))
+                    file_str.write(self.move_cursor(x, y // 2))
                     file_str.write(self.colorcode(self.color(pixels, dispose_pixels, x, y),
                                                   self.color(pixels, dispose_pixels, x, y + 1)))
-        file_str.write(self.move_cursor(width, padded_height / 2))
+        file_str.write(self.move_cursor(width, padded_height // 2))
         file_str.write(chr(27) + u"[0m")
         return file_str.getvalue()
 
@@ -245,9 +258,9 @@ class Shell8Bit(Shell):
         The returned value can be passed to colorcode().
         """
         # basically the opposite of what is done in 256colres.pl from the xterm source
-        r = (r - 55) / 40 if r > 55 else 0
-        g = (g - 55) / 40 if g > 55 else 0
-        b = (b - 55) / 40 if b > 55 else 0
+        r = (r - 55) // 40 if r > 55 else 0
+        g = (g - 55) // 40 if g > 55 else 0
+        b = (b - 55) // 40 if b > 55 else 0
         code = 16 + (r * 36) + (g * 6) + b
 
         return code
@@ -328,4 +341,4 @@ class Shell4Bit(Shell):
     @memoize
     def colorcode(bgcolor, fgcolor):
         return u"{0}[{1};{2}m▄ ".format(chr(27), Shell4Bit.color_value_4bit(*bgcolor) + 10, 
-                                     Shell4Bit.color_value_4bit(*fgcolor))
+                                        Shell4Bit.color_value_4bit(*fgcolor))
